@@ -17,10 +17,13 @@
 
 #include "lirc_driver.h"
 
+static const logchannel_t logchannel = LOG_DRIVER;
+
 static int init(void);
 static int deinit(void);
 static int decode(struct ir_remote* remote, struct decode_ctx_t* ctx);
 static char* rec(struct ir_remote* remotes);
+static int drvctl_func(unsigned int cmd, void* arg);
 
 static ir_code code, last_code;
 static snd_hwdep_t* hwdep;
@@ -41,23 +44,26 @@ const struct driver hw_alsa_usb = {
 	.send_func	= NULL,
 	.rec_func	= rec,
 	.decode_func	= decode,
-	.drvctl_func	= NULL,
+	.drvctl_func	= drvctl_func,
 	.readdata	= NULL,
-	.api_version	= 2,
-	.driver_version = "0.9.3",
-	.info		= "No info available."
+	.api_version	= 3,
+	.driver_version = "0.10.0",
+	.info		= "See file://" PLUGINDOCS "/alsa-usb.html",
+	.device_hint    = "drvctl"
 };
 
 const struct driver* hardwares[] = { &hw_alsa_usb, (const struct driver*)NULL };
 
 
-static const char* search_device(void)
+static int search_devices(glob_t* glob)
 {
 	int card, err;
 	snd_hwdep_info_t* info;
+	char name[36];
 
 	snd_hwdep_info_alloca(&info);
 	card = -1;
+	glob_t_init(glob);
 	while (snd_card_next(&card) >= 0 && card >= 0) {
 		char ctl_name[20];
 		snd_ctl_t* ctl;
@@ -71,60 +77,78 @@ static const char* search_device(void)
 		while (snd_ctl_hwdep_next_device(ctl, &device) >= 0 && device >= 0) {
 			snd_hwdep_info_set_device(info, device);
 			err = snd_ctl_hwdep_info(ctl, info);
-			if (err >= 0 && snd_hwdep_info_get_iface(info) == SND_HWDEP_IFACE_SB_RC) {
-				static char name[36];
-
+			if (err >= 0 &&
+			    snd_hwdep_info_get_iface(info) == SND_HWDEP_IFACE_SB_RC) {
 				sprintf(name, "hw:CARD=%d,DEV=%d", card, device);
-				snd_ctl_close(ctl);
-				return name;
+				glob_t_add_path(glob, name);
 			}
 		}
 		snd_ctl_close(ctl);
 	}
-	return NULL;
+	return 0;
 }
 
+
+static int drvctl_func(unsigned int cmd, void* arg)
+{
+	switch (cmd) {
+	case DRVCTL_GET_DEVICES:
+		return search_devices((glob_t*) arg);
+	case DRVCTL_FREE_DEVICES:
+		drv_enum_free((glob_t*) arg);
+		return 0;
+	default:
+		return DRV_ERR_NOT_IMPLEMENTED;
+	}
+}
+
+
 static int init(void)
 {
 	const char* device;
 	snd_hwdep_info_t* info;
 	struct pollfd pollfd;
 	int err;
+	glob_t glob;
+	static char my_device[36];
 
 	device = drv.device;
 	if (!device || !*device) {
-		device = search_device();
-		if (!device) {
-			logprintf(LIRC_ERROR, "device not found");
+		search_devices(&glob);
+		if (glob.gl_pathc == 0) {
+			log_error("device not found");
 			return 0;
 		}
+		strncpy(my_device, glob.gl_pathv[0], sizeof(my_device) - 1);
+		drv.device = my_device;
+		drv_enum_free(&glob);
 	}
 	err = snd_hwdep_open(&hwdep, device, SND_HWDEP_OPEN_READ);
 	if (err < 0) {
-		logprintf(LIRC_ERROR, "cannot open %s: %s", device, snd_strerror(err));
+		log_error("cannot open %s: %s", device, snd_strerror(err));
 		return 0;
 	}
 	snd_hwdep_info_alloca(&info);
 	err = snd_hwdep_info(hwdep, info);
 	if (err < 0) {
 		snd_hwdep_close(hwdep);
-		logprintf(LIRC_ERROR, "cannot get hwdep info: %s", snd_strerror(err));
+		log_error("cannot get hwdep info: %s", snd_strerror(err));
 		return 0;
 	}
 	if (snd_hwdep_info_get_iface(info) != SND_HWDEP_IFACE_SB_RC) {
 		snd_hwdep_close(hwdep);
-		logprintf(LIRC_ERROR, "%s is not a Sound Blaster remote control device", device);
+		log_error("%s is not a Sound Blaster remote control device", device);
 		return 0;
 	}
 	err = snd_hwdep_poll_descriptors(hwdep, &pollfd, 1);
 	if (err < 0) {
 		snd_hwdep_close(hwdep);
-		logprintf(LIRC_ERROR, "cannot get file descriptor: %s", snd_strerror(err));
+		log_error("cannot get file descriptor: %s", snd_strerror(err));
 		return 0;
 	}
 	if (err != 1) {
 		snd_hwdep_close(hwdep);
-		logprintf(LIRC_ERROR, "invalid number of file descriptors (%d): %s", err, snd_strerror(err));
+		log_error("invalid number of file descriptors (%d): %s", err, snd_strerror(err));
 		return 0;
 	}
 	drv.fd = pollfd.fd;
@@ -154,8 +178,8 @@ static char* rec(struct ir_remote* remotes)
 	repeat_flag = code == last_code && current.tv_sec - last_time.tv_sec <= 2
 		      && time_elapsed(&last_time, &current) <= 350000;
 	last_time = current;
-	LOGPRINTF(1, "code: %llx", (__u64)code);
-	LOGPRINTF(1, "repeat_flag: %d", repeat_flag);
+	log_trace("code: %llx", (uint64_t)code);
+	log_trace("repeat_flag: %d", repeat_flag);
 	return decode_all(remotes);
 }
 
