@@ -12,12 +12,18 @@
 # include <config.h>
 #endif
 
+#ifdef HAVE_LIBUDEV_H
+#include <libudev.h>
+#endif
+
 #include <dirent.h>
+#include <errno.h>
 #include <fcntl.h>
 #include <unistd.h>
 #include <limits.h>
 #include <signal.h>
 #include <stdio.h>
+#include <stdint.h>
 #include <stdlib.h>
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -25,10 +31,19 @@
 #include <sys/socket.h>
 #include <sys/un.h>
 
+#ifdef HAVE_KERNEL_LIRC_H
+#include <linux/lirc.h>
+#else
+#include "media/lirc.h"
+#endif
+
 #include "lirc_driver.h"
 
 
-static __u32 supported_send_modes[] = {
+
+static const logchannel_t logchannel = LOG_DRIVER;
+
+static uint32_t supported_send_modes[] = {
 	/* LIRC_CAN_SEND_LIRCCODE, */
 	/* LIRC_CAN_SEND_MODE2, this one would be very easy */
 	LIRC_CAN_SEND_PULSE,
@@ -36,7 +51,7 @@ static __u32 supported_send_modes[] = {
 	0
 };
 
-static __u32 supported_rec_modes[] = {
+static uint32_t supported_rec_modes[] = {
 	LIRC_CAN_REC_LIRCCODE,
 	LIRC_CAN_REC_MODE2,
 	/* LIRC_CAN_REC_PULSE, shouldn't be too hard */
@@ -52,10 +67,11 @@ static char* default_rec(struct ir_remote* remotes);
 static int default_ioctl(unsigned int cmd, void* arg);
 static lirc_t default_readdata(lirc_t timeout);
 static int my_open(const char* path);
+static int drvctl(unsigned int cmd, void* arg);
 
 
 
-static const const struct driver hw_default = {
+static const struct driver hw_default = {
 	.name		= "default",
 	.device		= LIRC_DRIVER_DEVICE,
 	.features	= 0,
@@ -69,11 +85,12 @@ static const const struct driver hw_default = {
 	.send_func	= default_send,
 	.rec_func	= default_rec,
 	.decode_func	= receive_decode,
-	.drvctl_func	= default_ioctl,
+	.drvctl_func	= drvctl,
 	.readdata	= default_readdata,
-	.api_version	= 2,
-	.driver_version = "0.9.3",
-	.info		= "No info available"
+	.api_version	= 3,
+	.driver_version = "0.10.0",
+	.info		= "See file://" PLUGINDOCS "/default.html",
+	.device_hint    = "drvctl",
 };
 
 
@@ -105,7 +122,8 @@ static int is_rc(const char* s)
 /*
  * Given a directory in /sys/class/rc, check if it contains
  * a file called device. If so, write "lirc" to the protocols
- * file in same directory.
+ * file in same directory unless the 'lirc' protocol already
+ * is enabled.
  *
  * rc_dir: directory specification like rc0, rc1, etc.
  * device: Device given to lirc,  like 'lirc0' (or /dev/lirc0).
@@ -114,27 +132,55 @@ static int is_rc(const char* s)
  */
 static int visit_rc(const char* rc_dir, const char* device)
 {
-	char path[64];
+	char path[512];
+	char buff[128];
+	char* enabled = NULL;
 	int fd;
+	int r;
 
 	snprintf(path, sizeof(path), "/sys/class/rc/%s", rc_dir);
 	if (access(path, F_OK) != 0) {
-		logprintf(LIRC_NOTICE, "Cannot open rc directory: %s", path);
+		log_notice("Cannot open rc directory: %s", path);
 		return -1;
 	}
 	snprintf(path, sizeof(path), "/sys/class/rc/%s/%s", rc_dir, device);
 	if (access(path, F_OK) != 0) {
-		logprintf(LIRC_DEBUG, "No device found: %s", path);
+		log_debug("No device found: %s", path);
 		return -1;
 	}
 	snprintf(path, sizeof(path), "/sys/class/rc/%s/protocols", rc_dir);
+	fd = open(path, O_RDONLY);
+	if (fd < 0) {
+		log_debug(
+			  "Cannot open protocol file: %s for read", path);
+		return -1;
+	}
+	r = read(fd, buff, sizeof(buff));
+	if (r < 0) {
+		log_debug("Cannot read from %s", path);
+		return -1;
+	}
+	if (strchr(buff, '[') != NULL) {
+		enabled = strchr(buff, '[') + 1;
+		if (strchr(buff, ']') != NULL)
+			*strchr(buff, ']') = '\0';
+		else
+			enabled = NULL;
+	}
+	if (enabled == NULL) {
+		log_warn("Cannot parse protocols %s", buff);
+	} else  if (strcmp(enabled, "lirc") == 0) {
+		log_info("[lirc] protocol is enabled");
+		return 0;
+	}
 	fd = open(path, O_WRONLY);
 	if (fd < 0) {
-		logprintf(LIRC_DEBUG, "Cannot open protocol file: %s", path);
+		log_debug(
+			  "Cannot open protocol file for write: %s", path);
 		return -1;
 	}
 	chk_write(fd, "lirc\n", 5);
-	logprintf(LIRC_NOTICE, "'lirc' written to protocols file %s", path);
+	log_notice("'lirc' written to protocols file %s", path);
 	close(fd);
 	return 0;
 }
@@ -153,7 +199,7 @@ static int set_rc_protocol(const char* device)
 		device = strrchr(device, '/') + 1;
 	dir = opendir("/sys/class/rc");
 	if (dir == NULL) {
-		logprintf(LIRC_NOTICE, "Cannot open /sys/class/rc\n");
+		log_notice("Cannot open /sys/class/rc\n");
 		return -1;
 	}
 	while ((ent = readdir(dir)) != NULL) {
@@ -167,9 +213,51 @@ static int set_rc_protocol(const char* device)
 }
 
 
+/** Log diagnostics if device cannot be accessed. */
+static void check_device(const char* device)
+{
+	if (device == NULL) {
+		log_error("default driver: NULL device");
+	} else {
+		if (access(device, R_OK | W_OK) != 0)
+			log_warn("Cannot access device: %s", drv.device);
+	}
+}
+
+/** Try to use an "auto" device specification. */
+static int get_auto_device(char* pathbuf, size_t size)
+{
+	const char* const PATTERN = "/dev/lirc*";
+	glob_t globbuf;
+
+	globbuf.gl_offs = 10;
+	glob(PATTERN, 0, NULL, &globbuf);
+	if (globbuf.gl_pathc == 0) {
+		log_error("No matching /dev/lirc device found for \"auto\"");
+		return  ENODEV;
+	}
+	if (globbuf.gl_pathc > 1) {
+		log_warn("Multiple /dev/lirc devices found for \"auto\"");
+	}
+	strncpy(pathbuf, globbuf.gl_pathv[0], size - 1);
+	globfree(&globbuf);
+	return 0;
+}
+
+
 int my_open(const char* path)
 {
-	default_open(path);
+	char pathbuf[128];
+	int r;
+
+	strncpy(pathbuf, path, sizeof(pathbuf) - 1);
+	if (strcmp(pathbuf, "auto") == 0) {
+		r = get_auto_device(pathbuf, sizeof(pathbuf));
+		if (r != 0)
+			return r;
+	}
+	default_open(pathbuf);
+	check_device(drv.device);
 	set_rc_protocol(drv.device);
 	return 0;
 }
@@ -183,31 +271,44 @@ int my_open(const char* path)
 int default_readdata(lirc_t timeout)
 {
 	int data, ret;
+	static int last_space = (lirc_t) 0;
 
 	if (!waitfordata((long)timeout))
 		return 0;
 
 	ret = read(drv.fd, &data, sizeof(data));
 	if (ret != sizeof(data)) {
-		logperror(LIRC_ERROR,
-			  "error reading from %s (ret %d, expected %d)",
+		log_perror_err("error reading from %s (ret %d, expected %d)",
 			  drv.device, ret, sizeof(data));
 		default_deinit();
 
 		return 0;
 	}
+	if (last_space == LIRC_SPACE(LIRC_VALUE_MASK) && LIRC_IS_SPACE(data)) {
+		/* Work around #172. */
+		if (!waitfordata((long)timeout))
+			return 0;
 
+		ret = read(drv.fd, &data, sizeof(data));
+		if (ret != sizeof(data)) {
+			log_perror_err(
+				"error reading from %s (got %d, expected %d)",
+				drv.device, ret, sizeof(data));
+			default_deinit();
+			return 0;
+		}
+	}
 	if (data == 0) {
 		static int data_warning = 1;
 
 		if (data_warning) {
-			logprintf(LIRC_WARNING,
-				  "read invalid data from device %s",
+			log_warn("read invalid data from device %s",
 				  drv.device);
 			data_warning = 0;
 		}
 		data = 1;
 	}
+	last_space = data;
 	return data;
 }
 
@@ -224,12 +325,12 @@ int default_init(void)
 	send_buffer_init();
 
 	if (set_rc_protocol(drv.device) != 0)
-		logprintf(LIRC_INFO, "Cannot configure the rc device for %s",
+		log_info("Cannot configure the rc device for %s",
 			  drv.device);
 
 	if (stat(drv.device, &s) == -1) {
-		logprintf(LIRC_ERROR, "could not get file information for %s", drv.device);
-		logperror(LIRC_ERROR, "default_init()");
+		log_error("could not get file information for %s", drv.device);
+		log_perror_err("default_init()");
 		return 0;
 	}
 
@@ -242,20 +343,20 @@ int default_init(void)
 
 		drv.fd = socket(AF_UNIX, SOCK_STREAM, 0);
 		if (drv.fd == -1) {
-			logprintf(LIRC_ERROR, "could not create socket");
-			logperror(LIRC_ERROR, "default_init()");
+			log_error("could not create socket");
+			log_perror_err("default_init()");
 			return 0;
 		}
 
 		if (connect(drv.fd, (struct sockaddr*)&addr, sizeof(addr)) == -1) {
-			logprintf(LIRC_ERROR, "could not connect to unix socket %s", drv.device);
-			logperror(LIRC_ERROR, "default_init()");
+			log_error("could not connect to unix socket %s", drv.device);
+			log_perror_err("default_init()");
 			default_deinit();
 			close(drv.fd);
 			return 0;
 		}
 
-		LOGPRINTF(1, "using unix socket lirc device");
+		log_trace("using unix socket lirc device");
 		drv.features = LIRC_CAN_REC_MODE2 | LIRC_CAN_SEND_PULSE;
 		drv.rec_mode = LIRC_MODE_MODE2; /* this might change in future */
 		drv.send_mode = LIRC_MODE_PULSE;
@@ -263,42 +364,42 @@ int default_init(void)
 	}
 	drv.fd = open(drv.device, O_RDWR);
 	if (drv.fd < 0) {
-		logprintf(LIRC_ERROR, "could not open %s", drv.device);
-		logperror(LIRC_ERROR, "default_init()");
+		log_error("could not open %s", drv.device);
+		log_perror_err("default_init()");
 		return 0;
 	}
 	if (S_ISFIFO(s.st_mode)) {
-		LOGPRINTF(1, "using defaults for the Irman");
+		log_trace("using defaults for the Irman");
 		drv.features = LIRC_CAN_REC_MODE2;
 		drv.rec_mode = LIRC_MODE_MODE2; /* this might change in future */
 		return 1;
 	} else if (!S_ISCHR(s.st_mode)) {
 		default_deinit();
-		logprintf(LIRC_ERROR, "%s is not a character device!!!", drv.device);
-		logperror(LIRC_ERROR, "something went wrong during installation");
+		log_error("%s is not a character device!!!", drv.device);
+		log_perror_err("something went wrong during installation");
 		return 0;
 	} else if (default_ioctl(LIRC_GET_FEATURES, &drv.features) == -1) {
-		logprintf(LIRC_ERROR, "could not get hardware features");
-		logprintf(LIRC_ERROR, "this device driver does not support the LIRC ioctl interface");
+		log_error("could not get hardware features");
+		log_error("this device driver does not support the LIRC ioctl interface");
 		if (major(s.st_rdev) == 13) {
-			logprintf(LIRC_ERROR, "did you mean to use the devinput driver instead of the %s driver?",
+			log_error("did you mean to use the devinput driver instead of the %s driver?",
 				  drv.name);
 		} else {
-			logprintf(LIRC_ERROR, "major number of %s is %lu", drv.device, (__u32)major(s.st_rdev));
-			logprintf(LIRC_ERROR, "make sure %s is a LIRC device and use a current version of the driver",
+			log_error("major number of %s is %lu", drv.device, (uint32_t)major(s.st_rdev));
+			log_error("make sure %s is a LIRC device and use a current version of the driver",
 				  drv.device);
 		}
 		default_deinit();
 		return 0;
 	}
 	if (!(LIRC_CAN_SEND(drv.features) || LIRC_CAN_REC(drv.features)))
-		LOGPRINTF(1, "driver supports neither sending nor receiving of IR signals");
+		log_trace("driver supports neither sending nor receiving of IR signals");
 	if (LIRC_CAN_SEND(drv.features) && LIRC_CAN_REC(drv.features)) {
-		LOGPRINTF(1, "driver supports both sending and receiving");
+		log_trace("driver supports both sending and receiving");
 	} else if (LIRC_CAN_SEND(drv.features)) {
-		LOGPRINTF(1, "driver supports sending");
+		log_trace("driver supports sending");
 	} else if (LIRC_CAN_REC(drv.features)) {
-		LOGPRINTF(1, "driver supports receiving");
+		log_trace("driver supports receiving");
 	}
 
 	/* set send/receive method */
@@ -311,7 +412,7 @@ int default_init(void)
 			}
 		}
 		if (supported_send_modes[i] == 0)
-			logprintf(LIRC_NOTICE, "the send method of the driver is not yet supported by lircd");
+			log_notice("the send method of the driver is not yet supported by lircd");
 	}
 	drv.rec_mode = 0;
 	if (LIRC_CAN_REC(drv.features)) {
@@ -322,24 +423,24 @@ int default_init(void)
 			}
 		}
 		if (supported_rec_modes[i] == 0)
-			logprintf(LIRC_NOTICE, "the receive method of the driver is not yet supported by lircd");
+			log_notice("the receive method of the driver is not yet supported by lircd");
 	}
 	if (drv.rec_mode == LIRC_MODE_MODE2) {
 		/* get resolution */
 		drv.resolution = 0;
 		if ((drv.features & LIRC_CAN_GET_REC_RESOLUTION)
 		    && (default_ioctl(LIRC_GET_REC_RESOLUTION, &drv.resolution) != -1))
-			LOGPRINTF(1, "resolution of receiver: %d", drv.resolution);
+			log_trace("resolution of receiver: %d", drv.resolution);
 
 	} else if (drv.rec_mode == LIRC_MODE_LIRCCODE) {
 		if (default_ioctl(LIRC_GET_LENGTH, (void*)&drv.code_length) == -1) {
-			logprintf(LIRC_ERROR, "could not get code length");
-			logperror(LIRC_ERROR, "default_init()");
+			log_error("could not get code length");
+			log_perror_err("default_init()");
 			default_deinit();
 			return 0;
 		}
 		if (drv.code_length > sizeof(ir_code) * CHAR_BIT) {
-			logprintf(LIRC_ERROR, "lircd can not handle %lu bit codes", drv.code_length);
+			log_error("lircd can not handle %lu bit codes", drv.code_length);
 			default_deinit();
 			return 0;
 		}
@@ -363,7 +464,7 @@ int default_deinit(void)
 static int write_send_buffer(int lirc)
 {
 	if (send_buffer_length() == 0) {
-		LOGPRINTF(1, "nothing to send");
+		log_trace("nothing to send");
 		return 0;
 	}
 	return write(lirc, send_buffer_data(), send_buffer_length() * sizeof(lirc_t));
@@ -378,28 +479,28 @@ int default_send(struct ir_remote* remote, struct ir_ncode* code)
 	if (drv.features & LIRC_CAN_SET_SEND_CARRIER) {
 		unsigned int freq;
 
-		freq = remote->freq == 0 ? DEFAULT_FREQ : remote->freq;
+		freq = remote->freq;
 		if (default_ioctl(LIRC_SET_SEND_CARRIER, &freq) == -1) {
-			logprintf(LIRC_ERROR, "could not set modulation frequency");
-			logperror(LIRC_ERROR, NULL);
+			log_error("could not set modulation frequency");
+			log_perror_err(NULL);
 			return 0;
 		}
 	}
 	if (drv.features & LIRC_CAN_SET_SEND_DUTY_CYCLE) {
 		unsigned int duty_cycle;
 
-		duty_cycle = remote->duty_cycle == 0 ? 50 : remote->duty_cycle;
+		duty_cycle = get_duty_cycle(remote);
 		if (default_ioctl(LIRC_SET_SEND_DUTY_CYCLE, &duty_cycle) == -1) {
-			logprintf(LIRC_ERROR, "could not set duty cycle");
-			logperror(LIRC_ERROR, NULL);
+			log_error("could not set duty cycle");
+			log_perror_err(NULL);
 			return 0;
 		}
 	}
 	if (!send_buffer_put(remote, code))
 		return 0;
 	if (write_send_buffer(drv.fd) == -1) {
-		logprintf(LIRC_ERROR, "write failed");
-		logperror(LIRC_ERROR, NULL);
+		log_error("write failed");
+		log_perror_err(NULL);
 		return 0;
 	}
 	return 1;
@@ -418,3 +519,29 @@ int default_ioctl(unsigned int cmd, void* arg)
 {
 	return ioctl(drv.fd, cmd, arg);
 }
+
+
+static int list_devices(glob_t* glob)
+{
+	static const struct drv_enum_udev_what what[] = {
+		{.subsystem = "lirc" },
+		{0}
+	};
+	return drv_enum_udev(glob, what);
+}
+
+
+static int drvctl(unsigned int cmd, void* arg)
+{
+	switch (cmd) {
+	case DRVCTL_GET_DEVICES:
+		return list_devices((glob_t*) arg);
+	case DRVCTL_FREE_DEVICES:
+		drv_enum_free((glob_t*) arg);
+		return 0;
+	case LIRC_SET_TRANSMITTER_MASK:
+		return default_ioctl(LIRC_SET_TRANSMITTER_MASK, arg);
+	default:
+		return DRV_ERR_NOT_IMPLEMENTED;
+	}
+}
