@@ -28,6 +28,7 @@
 #endif
 
 #include <stdio.h>
+#include <stdint.h>
 #include <stdlib.h>
 #include <stdarg.h>
 #include <fcntl.h>
@@ -44,6 +45,8 @@
 
 #define NUMBYTES 6
 
+static const logchannel_t logchannel = LOG_DRIVER;
+
 static uirt2_t* dev;
 static lirc_t rec_buf[200];
 static int rec_rptr;
@@ -82,9 +85,10 @@ const struct driver hw_uirt2_raw = {
 	.decode_func	= uirt2_raw_decode,
 	.drvctl_func	= NULL,
 	.readdata	= uirt2_raw_readdata,
-	.api_version	= 2,
-	.driver_version = "0.9.3",
-	.info		= "No info available"
+	.api_version	= 3,
+	.driver_version = "0.10.0",
+	.info		= "No info available",
+	.device_hint    = "/dev/tty[0-9]*",
 };
 
 const struct driver hw_usb_uirt_raw = {
@@ -103,9 +107,10 @@ const struct driver hw_usb_uirt_raw = {
 	.decode_func	= uirt2_raw_decode,
 	.drvctl_func	= NULL,
 	.readdata	= uirt2_raw_readdata,
-	.api_version	= 2,
-	.driver_version = "0.9.3",
-	.info		= "No info available"
+	.api_version	= 3,
+	.driver_version = "0.10.0",
+	.info		= "No info available",
+	.device_hint    = "/dev/tty[0-9]*",
 };
 
 const struct driver* hardwares[] = { &hw_usb_uirt_raw, &hw_uirt2_raw, NULL };
@@ -117,14 +122,14 @@ static int queue_put(lirc_t data)
 {
 	int next = (rec_wptr + 1) % rec_size;
 
-	LOGPRINTF(3, "queue_put: %d", data);
+	log_trace2("queue_put: %d", data);
 
 	if (next != rec_rptr) {
 		rec_buf[rec_wptr] = data;
 		rec_wptr = next;
 		return 0;
 	}
-	logprintf(LIRC_ERROR, "uirt2_raw: queue full");
+	log_error("uirt2_raw: queue full");
 	return -1;
 }
 
@@ -133,11 +138,11 @@ static int queue_get(lirc_t* pdata)
 	if (rec_wptr != rec_rptr) {
 		*pdata = rec_buf[rec_rptr];
 		rec_rptr = (rec_rptr + 1) % rec_size;
-		LOGPRINTF(3, "queue_get: %d", *pdata);
+		log_trace2("queue_get: %d", *pdata);
 
 		return 0;
 	}
-	logprintf(LIRC_ERROR, "uirt2_raw: queue empty");
+	log_error("uirt2_raw: queue empty");
 	return -1;
 }
 
@@ -156,11 +161,11 @@ static int uirt2_raw_decode(struct ir_remote* remote, struct decode_ctx_t* ctx)
 {
 	int res;
 
-	LOGPRINTF(1, "uirt2_raw_decode: enter");
+	log_trace("uirt2_raw_decode: enter");
 
 	res = receive_decode(remote, ctx);
 
-	LOGPRINTF(1, "uirt2_raw_decode: %d", res);
+	log_trace("uirt2_raw_decode: %d", res);
 
 	return res;
 }
@@ -173,7 +178,7 @@ static lirc_t uirt2_raw_readdata(lirc_t timeout)
 		lirc_t data = uirt2_read_raw(dev, timeout);
 
 		if (!data) {
-			LOGPRINTF(1, "uirt2_raw_readdata failed");
+			log_trace("uirt2_raw_readdata failed");
 			return 0;
 		}
 
@@ -182,7 +187,7 @@ static lirc_t uirt2_raw_readdata(lirc_t timeout)
 
 	queue_get(&data);
 
-	LOGPRINTF(1, "uirt2_raw_readdata %d %d", !!(data & PULSE_BIT), data & PULSE_MASK);
+	log_trace("uirt2_raw_readdata %d %d", !!(data & PULSE_BIT), data & PULSE_MASK);
 
 	return data;
 }
@@ -192,19 +197,19 @@ static int uirt2_raw_init(void)
 	int version;
 
 	if (!tty_create_lock(drv.device)) {
-		logprintf(LIRC_ERROR, "uirt2_raw: could not create lock files");
+		log_error("uirt2_raw: could not create lock files");
 		return 0;
 	}
 
 	drv.fd = open(drv.device, O_RDWR | O_NONBLOCK | O_NOCTTY);
 	if (drv.fd < 0) {
-		logprintf(LIRC_ERROR, "uirt2_raw: could not open %s", drv.device);
+		log_error("uirt2_raw: could not open %s", drv.device);
 		tty_delete_lock();
 		return 0;
 	}
 
 	if (!tty_reset(drv.fd)) {
-		logprintf(LIRC_ERROR, "uirt2_raw: could not reset tty");
+		log_error("uirt2_raw: could not reset tty");
 		close(drv.fd);
 		tty_delete_lock();
 		return 0;
@@ -214,21 +219,21 @@ static int uirt2_raw_init(void)
 	usleep(100 * 1000);
 
 	if (!tty_setbaud(drv.fd, 115200)) {
-		logprintf(LIRC_ERROR, "uirt2_raw: could not set baud rate");
+		log_error("uirt2_raw: could not set baud rate");
 		close(drv.fd);
 		tty_delete_lock();
 		return 0;
 	}
 
 	if (!tty_setcsize(drv.fd, 8)) {
-		logprintf(LIRC_ERROR, "uirt2_raw: could not set csize");
+		log_error("uirt2_raw: could not set csize");
 		close(drv.fd);
 		tty_delete_lock();
 		return 0;
 	}
 
 	if (!tty_setrtscts(drv.fd, 1)) {
-		logprintf(LIRC_ERROR, "uirt2_raw: could not enable hardware flow");
+		log_error("uirt2_raw: could not enable hardware flow");
 		close(drv.fd);
 		tty_delete_lock();
 		return 0;
@@ -236,14 +241,14 @@ static int uirt2_raw_init(void)
 
 	dev = uirt2_init(drv.fd);
 	if (dev == NULL) {
-		logprintf(LIRC_ERROR, "uirt2_raw: No UIRT2 device found at %s", drv.device);
+		log_error("uirt2_raw: No UIRT2 device found at %s", drv.device);
 		close(drv.fd);
 		tty_delete_lock();
 		return 0;
 	}
 
 	if (uirt2_setmoderaw(dev) < 0) {
-		logprintf(LIRC_ERROR, "uirt2_raw: could not set raw mode");
+		log_error("uirt2_raw: could not set raw mode");
 		uirt2_raw_deinit();
 		return 0;
 	}
@@ -254,7 +259,7 @@ static int uirt2_raw_init(void)
 	}
 	if (version >= 0x0905) {
 		if (!tty_setdtr(drv.fd, 0)) {
-			logprintf(LIRC_ERROR, "uirt2_raw: could not set DTR");
+			log_error("uirt2_raw: could not set DTR");
 			uirt2_raw_deinit();
 			return 0;
 		}
@@ -274,6 +279,8 @@ static int uirt2_raw_deinit(void)
 {
 	int version;
 
+	if (uirt2_setmodeuir(dev) < 0)
+		log_warn("uirt2_raw: could not set uir mode");
 	if (uirt2_getversion(dev, &version) >= 0 && version >= 0x0905)
 		tty_setdtr(drv.fd, 1);
 	uirt2_uninit(dev);
@@ -286,8 +293,8 @@ static int uirt2_raw_deinit(void)
 
 static char* uirt2_raw_rec(struct ir_remote* remotes)
 {
-	LOGPRINTF(1, "uirt2_raw_rec");
-	LOGPRINTF(1, "uirt2_raw_rec: %p", remotes);
+	log_trace("uirt2_raw_rec");
+	log_trace("uirt2_raw_rec: %p", remotes);
 
 	if (!rec_buffer_clear())
 		return NULL;
@@ -321,23 +328,23 @@ static int uirt2_send(struct ir_remote* remote, struct ir_ncode* code)
 	signals = send_buffer_data();
 
 	if (length <= 0 || signals == NULL) {
-		LOGPRINTF(1, "nothing to send");
+		log_trace("nothing to send");
 		return 0;
 	}
 
 
-	LOGPRINTF(1, "Trying REMSTRUC1 transmission");
+	log_trace("Trying REMSTRUC1 transmission");
 	res = uirt2_send_mode2_struct1(dev, remote, signals, length);
 	if (!res && (length < 48)) {
-		LOGPRINTF(1, "Using RAW transission");
+		log_trace("Using RAW transission");
 		res = uirt2_send_mode2_raw(dev, remote, signals, length);
 	}
 
-	if (!res)
-		logprintf(LIRC_ERROR, "uirt2_send: remote not supported");
-	else
-		LOGPRINTF(1, "uirt2_send: succeeded");
-
+	if (!res) {
+		log_error("uirt2_send: remote not supported");
+	} else {
+		log_trace("uirt2_send: succeeded");
+	}
 	/*
 	 * Some devices send the sequence in the background.  Wait for
 	 * the sequence to complete before returning in order to avoid
@@ -366,7 +373,7 @@ static int uirt2_send_mode2_raw(uirt2_t*		dev,
 	int res;
 	int repeats = 1;
 
-	LOGPRINTF(1, "uirt2_send_mode2_raw %d %p", length, buf);
+	log_trace("uirt2_send_mode2_raw %d %p", length, buf);
 
 	tmp[0] = 0;
 	tmp[1] = 0;
@@ -385,7 +392,7 @@ static int uirt2_send_mode2_raw(uirt2_t*		dev,
 				val = 0;
 			}
 			if (dest - 2 > 48) {
-				logprintf(LIRC_ERROR, "uirt2_raw: too long RAW transmission %d > 48", dest - 2);
+				log_error("uirt2_raw: too long RAW transmission %d > 48", dest - 2);
 				return 0;
 			}
 		}
@@ -399,7 +406,7 @@ static int uirt2_send_mode2_raw(uirt2_t*		dev,
 	if (!res)
 		return 0;
 
-	LOGPRINTF(1, "uirt2_send_mode2_raw exit");
+	log_trace("uirt2_send_mode2_raw exit");
 	return 1;
 }
 
@@ -428,18 +435,18 @@ static int calc_data_bit(struct ir_remote* remote, int table[], int table_len, i
 		if (table[i] == 0) {
 			table[i] = signal / tUnit;
 
-			LOGPRINTF(2, "table[%d] = %d\n", i, table[i]);
+			log_trace1("table[%d] = %d\n", i, table[i]);
 
 			return i;
 		}
 
 		if (expect(remote, signal, table[i] * tUnit)) {
-			LOGPRINTF(2, "expect %d, table[%d] = %d\n", signal / tUnit, i, table[i]);
+			log_trace1("expect %d, table[%d] = %d\n", signal / tUnit, i, table[i]);
 			return i;
 		}
 	}
 
-	LOGPRINTF(2, "Couldn't find %d\n", signal / tUnit);
+	log_trace1("Couldn't find %d\n", signal / tUnit);
 
 	return -1;
 }
@@ -469,7 +476,7 @@ static int uirt2_send_mode2_struct1(uirt2_t*		dev,
 	res = uirt2_getversion(dev, &version);
 	if (res < 0)
 		return res;
-	logprintf(LIRC_INFO, "uirt2_raw: UIRT version %04x", version);
+	log_info("uirt2_raw: UIRT version %04x", version);
 	freq = remote->freq;
 	if (freq == 0)
 		freq = DEFAULT_FREQ;
@@ -487,7 +494,7 @@ static int uirt2_send_mode2_struct1(uirt2_t*		dev,
 		int len = buf[i] / tUnit;
 
 		if (len > UCHAR_MAX) {
-			LOGPRINTF(0, "signal too long for transmission %lu", (__u32)buf[i]);
+			log_trace("signal too long for transmission %lu", (uint32_t)buf[i]);
 			return 0;
 		}
 		if (i == 0) {
@@ -522,7 +529,7 @@ static int uirt2_send_mode2_struct1(uirt2_t*		dev,
 		}
 
 		if (i - 2 > UIRT2_MAX_BITS) {
-			logprintf(LIRC_ERROR, "uirt2_raw: UIRT tried to send %d bits, max is %d", length - 2,
+			log_error("uirt2_raw: UIRT tried to send %d bits, max is %d", length - 2,
 				  UIRT2_MAX_BITS);
 
 			return 0;
@@ -532,7 +539,7 @@ static int uirt2_send_mode2_struct1(uirt2_t*		dev,
 		bits++;
 	}
 
-	LOGPRINTF(2, "bits %d", bits);
+	log_trace1("bits %d", bits);
 
 	rem.bISDlyHi = remote->min_remaining_gap / tUnit / 256;
 	rem.bISDlyLo = (remote->min_remaining_gap / tUnit) & 255;
